@@ -119,32 +119,33 @@ check_menu_sens (gboolean scanning)
 			gtk_main_iteration ();
 
 		set_statusbar (_("Scanning..."));
-		set_glade_widget_sens ("expand_all", TRUE);
-		set_glade_widget_sens ("collapse_all", TRUE);
+		set_ui_action_sens ("expand_all", TRUE);
+		set_ui_action_sens ("collapse_all", TRUE);
 	}
 
-	set_glade_widget_sens ("tbscanhome", !scanning);
-	set_glade_widget_sens ("tbscanall", !scanning);
-	set_glade_widget_sens ("tbscandir", !scanning);
-	set_glade_widget_sens ("menuscanhome", !scanning);
-	set_glade_widget_sens ("menuallfs", !scanning);
-	set_glade_widget_sens ("menuscandir", !scanning);
-	set_glade_widget_sens ("tbstop", scanning);
-	set_glade_widget_sens ("tbrescan", !scanning && current_location != NULL);
-	set_glade_widget_sens ("menustop", scanning);
-	set_glade_widget_sens ("menurescan", !scanning && current_location != NULL);
-	set_glade_widget_sens ("preferenze1", !scanning);
-	set_glade_widget_sens ("menu_scan_rem", !scanning);
-	set_glade_widget_sens ("tb_scan_remote", !scanning);
-	set_glade_widget_sens ("ck_allocated",
-			       !scanning &&
-			       baobab.is_local && !g_noactivescans);
+	set_ui_action_sens ("menuscanhome", !scanning);
+	set_ui_action_sens ("menuallfs", !scanning);
+	set_ui_action_sens ("menuscandir", !scanning);
+	set_ui_action_sens ("menustop", scanning);
+	set_ui_action_sens ("menurescan", !scanning && current_location != NULL);
+	set_ui_action_sens ("preferenze1", !scanning);
+	set_ui_action_sens ("menu_scan_rem", !scanning);
+	set_ui_action_sens ("ck_allocated",
+			    !scanning &&
+			    baobab.is_local && !g_noactivescans);
+
+	set_ui_widget_sens ("tbscanhome", !scanning);
+	set_ui_widget_sens ("tbscanall", !scanning);
+	set_ui_widget_sens ("tbscandir", !scanning);
+	set_ui_widget_sens ("tbstop", scanning);
+	set_ui_widget_sens ("tbrescan", !scanning && current_location != NULL);
+	set_ui_widget_sens ("tb_scan_remote", !scanning);
 }
 
 void
 baobab_scan_location (GFile *file)
 {
-	GtkWidget *ck_allocated;
+	GtkToggleAction *ck_allocated;
 
 	if (!baobab_check_dir (file))
 		return;
@@ -166,15 +167,14 @@ baobab_scan_location (GFile *file)
 
 	/* check if the file system is local or remote */
 	baobab.is_local = scan_is_local (file);
-	ck_allocated = glade_xml_get_widget (baobab.main_xml, "ck_allocated");
+	ck_allocated = GTK_TOGGLE_ACTION (gtk_builder_get_object (baobab.main_ui, "ck_allocated"));
 	if (!baobab.is_local) {
-		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (ck_allocated),
-						FALSE);
-		gtk_widget_set_sensitive (ck_allocated, FALSE);
+		gtk_toggle_action_set_active (ck_allocated, FALSE);
+		gtk_action_set_sensitive (GTK_ACTION (ck_allocated), FALSE);
 		baobab.show_allocated = FALSE;
 	}
 	else {
-		gtk_widget_set_sensitive (ck_allocated, TRUE);
+		gtk_action_set_sensitive (GTK_ACTION (ck_allocated), TRUE);
 	}
 
 	baobab_scan_execute (file);
@@ -470,7 +470,7 @@ baobab_is_excluded_location (GFile *file)
 void
 set_toolbar_visible (gboolean visible)
 {
-	GtkWidget *menu;
+	GtkToggleAction *action;
 
 	if (visible)
 		gtk_widget_show (baobab.toolbar);
@@ -478,14 +478,14 @@ set_toolbar_visible (gboolean visible)
 		gtk_widget_hide (baobab.toolbar);
 
 	/* make sure the check menu item is consistent */
-	menu = glade_xml_get_widget (baobab.main_xml, "view_tb");
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu), visible);
+	action = GTK_TOGGLE_ACTION (gtk_builder_get_object (baobab.main_ui, "view_tb"));
+	gtk_toggle_action_set_active (action, visible);
 }
 
 void
 set_statusbar_visible (gboolean visible)
 {
-	GtkWidget *menu;
+	GtkToggleAction *action;
 
 	if (visible)
 		gtk_widget_show (baobab.statusbar);
@@ -493,8 +493,8 @@ set_statusbar_visible (gboolean visible)
 		gtk_widget_hide (baobab.statusbar);
 
 	/* make sure the check menu item is consistent */
-	menu = glade_xml_get_widget (baobab.main_xml, "view_sb");
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu), visible);
+	action = GTK_TOGGLE_ACTION (gtk_builder_get_object (baobab.main_ui, "view_sb"));
+	gtk_toggle_action_set_active (action, visible);
 }
 
 void
@@ -568,7 +568,7 @@ baobab_create_toolbar (void)
 	GtkToolItem *separator;
 	gboolean visible;
 
-	toolbar = glade_xml_get_widget (baobab.main_xml, "toolbar1");
+	toolbar = GTK_WIDGET (gtk_builder_get_object (baobab.main_ui, "toolbar1"));
 	if (toolbar == NULL) {
 		g_printerr ("Could not build toolbar\n");
 		return;
@@ -606,8 +606,8 @@ baobab_create_statusbar (void)
 {
 	gboolean visible;
 
-	baobab.statusbar = glade_xml_get_widget (baobab.main_xml,
-						 "statusbar1");
+	baobab.statusbar = GTK_WIDGET (gtk_builder_get_object (baobab.main_ui,
+							       "statusbar1"));
 	if (baobab.statusbar == NULL) {
 		g_printerr ("Could not build statusbar\n");
 		return;
@@ -674,17 +674,98 @@ sanity_check_excluded_locations (void)
 }
 
 static void
-baobab_init (void)
+volume_changed (GVolumeMonitor *volume_monitor,
+                GVolume        *volume,
+                gpointer        user_data)
+{
+	/* filesystem has changed (mounted or unmounted device) */
+	baobab_get_filesystem (&g_fs);
+	set_label_scan (&g_fs);
+	show_label ();
+}
+
+static void
+home_contents_changed (GFileMonitor      *file_monitor,
+		       GFile             *child,
+		       GFile             *other_file,
+		       GFileMonitorEvent  event_type,
+		       gpointer           user_data)
+
+{
+	gchar *excluding;
+
+	if (!baobab.bbEnableHomeMonitor)
+		return;
+
+	if (baobab.CONTENTS_CHANGED_DELAYED)
+		return;
+
+	excluding = g_file_get_basename (child);
+	if (strcmp (excluding, ".recently-used") == 0   ||
+	    strcmp (excluding, ".gnome2_private") == 0  ||
+	    strcmp (excluding, ".xsession-errors") == 0 ||
+	    strcmp (excluding, ".bash_history") == 0    ||
+	    strcmp (excluding, ".gconfd") == 0) {
+		g_free (excluding);
+		return;
+	}
+	g_free (excluding);
+
+	baobab.CONTENTS_CHANGED_DELAYED = TRUE;
+}
+
+static void
+monitor_volume (void)
+{
+	monitor_vol = g_volume_monitor_get ();
+	g_signal_connect (monitor_vol, "volume_changed",
+			  G_CALLBACK (volume_changed), NULL);
+}
+
+static void
+monitor_home_dir (void)
 {
-	GSList *uri_list;
 	GFile *file;
 	GError *error = NULL;
 	monitor_home = NULL;
 
-	/* Load Glade */
-	baobab.main_xml = glade_xml_new (BAOBAB_GLADE_FILE,
-					 "baobab_window", NULL);
-	glade_xml_signal_autoconnect (baobab.main_xml);
+	file = g_file_new_for_path (g_get_home_dir ());
+	monitor_home = g_file_monitor_directory (file, 0, NULL, &error);
+	g_object_unref (file);
+
+	if (!monitor_home) {
+		message (_("Could not initialize monitoring"),
+			 _("Changes to your home folder will not be monitored."),
+			 GTK_MESSAGE_WARNING, NULL);
+		g_print ("homedir:%s\n", error->message);
+		g_error_free (error);
+	}
+	else {
+		g_signal_connect (monitor_home,
+				  "changed",
+				  G_CALLBACK (home_contents_changed),
+				  NULL);
+	}
+}
+
+static void
+baobab_init (void)
+{
+	GSList *uri_list;
+	GError *error = NULL;
+
+	/* Load the UI */
+	baobab.main_ui = gtk_builder_new ();
+	gtk_builder_add_from_file (baobab.main_ui, BAOBAB_UI_FILE, &error);
+
+	if (error) {
+		g_object_unref (baobab.main_ui);
+		g_critical ("Unable to load the user interface file: %s", error->message);
+		g_error_free (error);
+		exit (1);
+	}
+
+	gtk_builder_connect_signals (baobab.main_ui, NULL);
 
 	/* Misc */
 	baobab.label_scan = NULL;
@@ -716,36 +797,17 @@ baobab_init (void)
 
 	sanity_check_excluded_locations ();
 
-	baobab.bbEnableHomeMonitor = gconf_client_get_bool (baobab.gconf_client,
-							    PROPS_ENABLE_HOME_MONITOR,
-							    NULL);
-
 	baobab_create_toolbar ();
 
 	baobab_create_statusbar ();
 
-	/* start monitoring */
-	monitor_vol = g_volume_monitor_get ();
-	g_signal_connect (monitor_vol, "volume_changed",
-			  G_CALLBACK (volume_changed), NULL);
+	monitor_volume ();
 
-	file = g_file_new_for_path (g_get_home_dir ());
-	monitor_home = g_file_monitor_directory (file, 0, NULL, &error);
-	g_object_unref (file);
+	baobab.bbEnableHomeMonitor = gconf_client_get_bool (baobab.gconf_client,
+							    PROPS_ENABLE_HOME_MONITOR,
+							    NULL);
 
-	if (!monitor_home) {
-		message (_("Could not initialize monitoring"),
-			 _("Changes to your home folder will not be monitored."),
-			 GTK_MESSAGE_WARNING, NULL);
-		g_print ("homedir:%s\n", error->message);
-		g_error_free (error);
-	}
-	else {
-		g_signal_connect (monitor_home,
-				  "changed",
-				  G_CALLBACK (contents_changed_cb),
-				  NULL);
-	}
+	monitor_home_dir ();
 }
 
 static void
@@ -833,7 +895,7 @@ initialize_ringschart (void)
 	gtk_frame_set_label_align (GTK_FRAME (chart_frame), 0.0, 0.0);
 	gtk_frame_set_shadow_type (GTK_FRAME (chart_frame), GTK_SHADOW_IN);
 
-	hpaned_main = glade_xml_get_widget (baobab.main_xml, "hpaned_main");
+	hpaned_main = GTK_WIDGET (gtk_builder_get_object (baobab.main_ui, "hpaned_main"));
 	gtk_paned_pack2 (GTK_PANED (hpaned_main),
 			 chart_frame, TRUE, TRUE);
 	gtk_paned_set_position (GTK_PANED (hpaned_main), 480);
@@ -849,7 +911,7 @@ initialize_ringschart (void)
 			  "changed",
 			  G_CALLBACK (on_chart_type_change), NULL);
 
-	hbox1 = glade_xml_get_widget (baobab.main_xml, "hbox1");
+	hbox1 = GTK_WIDGET (gtk_builder_get_object (baobab.main_ui, "hbox1"));
 	gtk_container_add (GTK_CONTAINER (hbox1), baobab.chart_type_combo);
 	gtk_box_set_spacing (GTK_BOX (hbox1), 50);
 	gtk_box_set_child_packing (GTK_BOX (hbox1),
@@ -913,10 +975,6 @@ initialize_ringschart (void)
 static gboolean
 start_proc_on_command_line (GFile *file)
 {
-	if (baobab_is_excluded_location (file)) {
-		message("", _("Cannot check an excluded folder!"), GTK_MESSAGE_INFO, baobab.window);
-		return FALSE;
-		}
 		
 	baobab_scan_location (file);
 
@@ -977,19 +1035,17 @@ main (int argc, char *argv[])
 	set_label_scan (&g_fs);
 	show_label ();
 
-	baobab.window = glade_xml_get_widget (baobab.main_xml, "baobab_window");
+	baobab.window = GTK_WIDGET (gtk_builder_get_object (baobab.main_ui, "baobab_window"));
 	gtk_window_set_position (GTK_WINDOW (baobab.window),
 				 GTK_WIN_POS_CENTER);
 
 	baobab.tree_view = create_directory_treeview ();
 
-	set_glade_widget_sens ("menurescan",FALSE);
+	set_ui_action_sens ("menurescan", FALSE);
 
 	/* set allocated space checkbox */
-	gtk_check_menu_item_set_active ((GtkCheckMenuItem *)
-				      glade_xml_get_widget (baobab.
-							    main_xml,
-							    "ck_allocated"),
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (gtk_builder_get_object (baobab.main_ui,
+							 "ck_allocated")),
 				      baobab.show_allocated);
 
 	gtk_widget_show (baobab.window);
